/*
 * Senses_BRD.c
 *
 * Created: 26/01/2016 19:06:10
 * Author : Ji ech 
 */ 

#include <avr/io.h>
#define F_CPU 8000000 //CPU clock frequency is 8MHz
#include <util/delay.h>

//Configuration 
#define RC5_ADDRESS 26 //Address of the RC-5 transmitter which will be used. Other algorithms and addresses won't be carried.
#define PERIOD 104.1666 //Period of the emulated UART (T=104.1666->9600 B) 
#define PERIOD_RC5 889 //Period of the RC-5

unsigned char tmpl;
unsigned char c;
signed char i;

struct{ //RC-5 receiver data structure
	unsigned char address; //Received address
	unsigned char data; //Received data
	unsigned char toggle; //Received toggle bit (unused- reserved for next versions)
	unsigned char error; //Error flag
	unsigned char tmp; //Temporarily data storage
	}IR;

//Read byte from UART
//Returns null if stop bit hasn't been detected
unsigned char UART_byte_read(){
	
unsigned char rx_byte=0; //Rx buffer
unsigned char i; //Iterator
while((PINB&0b00010000)!=0){} //Wait on start bit
_delay_us(PERIOD/2);
//If start bit has been detected
if ((PINB&0b00010000)==0){
for(i=0; i!=8; i++){
	_delay_us(PERIOD-2);
	if((PINB&0b00010000)==0){
		rx_byte&=~(1<<i);		
	}else{
		rx_byte|=(1<<i);
	}	
}
}
_delay_us(PERIOD);
//Stop bit detection
if((PINB&0b00010000)==0){
	rx_byte=0; //Return NULL if stop bit hasn't been detected
}
return rx_byte;
}

//Read byte from the emulated UART
//Returns null if stop bit hasn't been detected
unsigned char UART_byte_read_nowait(){
	
	unsigned char rx_byte=0; //Rx buffer
	unsigned char i; //Iterator
	_delay_us(PERIOD/2);
	//If start bit has been detected
	if ((PINB&0b00010000)==0){
		for(i=0; i!=8; i++){
			_delay_us(PERIOD-2);
			if((PINB&0b00010000)==0){
				rx_byte&=~(1<<i);
				}else{
				rx_byte|=(1<<i);
			}
		}
	}
	_delay_us(PERIOD);
	//Stop bit detection
	if((PINB&0b00010000)==0){
		rx_byte=0; //Return NULL if stop bit hasn't been detected
	}
	return rx_byte;
}

//Send byte via emulated UART (TxD:PB3)
//9600-1-8-N
void UART_byte_write(unsigned char tx_byte){
	unsigned char i; //Iterator
	unsigned char tmp; //Temporary storage of data
	
	//Start bit
	PORTB&=0b11110111;
	_delay_us(PERIOD);
	
	//Send data (LSB at first)
	for(i=0; i!=8; i++){
		
		tmp=tx_byte>>i;
		tmp&=0b00000001;
		if(tmp==0){
			PORTB&=0b11110111;
			}else{
			PORTB|=0b00001000;
		}
		_delay_us(PERIOD-2);
	}
	
	//Stop bit
	
	PORTB|=0b00001000;
	_delay_us(PERIOD);
	
}

//Enable buzzer for (argument) 0: short time (8ms) | Any other value: long time (22ms) 
void buzzer(unsigned char time){
PORTB|=0b00000010;
_delay_ms(8);
if(time!=0){
_delay_ms(22);
}
PORTB&=0b11111101;
}


// Read ADC1 
unsigned char read_adc()
{
	ADMUX=0b00100001; //Select ADC in 1
	_delay_us(15);
	ADCSRA|=0x40;
	while ((ADCSRA & 0x10)==0);
	ADCSRA|=0x10;
	return ADCH;
}

int main(void)
{
	DDRB=0b00001010;
	PORTB=0b00011000;
	
	// ADC fs=62.5kHz; bipolar mode; ADC stopped mode; 8-bit mode
	DIDR0&=0x03;
	ADMUX=0x20;
	ADCSRA=0x87;
	ADCSRB&=0x5F;

	asm("sei");
    while(1)
	{
		if((PINB&0b00010000)==0){ //If is detected start bit
				switch(UART_byte_read_nowait()){ //Receive data via UART
				
				case '!':
				UART_byte_write('_'); //Answer '_'
				if(UART_byte_read()=='#'){ //If is SENSE BRD forced to return e_sense value
					UART_byte_write(read_adc()); //Return e_sense value (8bit)
				}
				break;
				
				case '%': //Enable buzzer for a short time
				buzzer(0);
				break;
				
				case '/': //Enable buzzer for a long time
				buzzer(1);
				break;
				
				}
		}else if((PINB&1)==0){
			
			_delay_us(400); 
			
			for(i=0; i!=2; i++){
				if ((PINB&1)==1) IR.error=1;
				_delay_us(PERIOD_RC5);
				if ((PINB&1)==0) IR.error=1;
				_delay_us(PERIOD_RC5);
			}
			
			IR.toggle=(PINB&1);
			_delay_us(PERIOD_RC5);
			IR.toggle|=((PINB&1)<<1);
			switch(IR.toggle){
				case 1:
				IR.toggle=1;
				break;
				
				case 2:
				IR.toggle=0;
				break;
				
				default:
				IR.error=1;
				break;
			}
			_delay_us(PERIOD_RC5);
			
			for(i=4; i!=-1; i--){
				IR.tmp=(PINB&1);
				_delay_us(PERIOD_RC5);
				IR.tmp|=((PINB&1)<<1);
				switch(IR.tmp){
					case 1:
					break;
					
					case 2:
					IR.address|=(1<<i);
					break;
					
					default:
					IR.error=1;
					break;
				}
				_delay_us(PERIOD_RC5);
			}
			
			for(i=5; i!=-1; i--){
				IR.tmp=(PINB&1);
				_delay_us(PERIOD_RC5);
				IR.tmp|=((PINB&1)<<1);
				switch(IR.tmp){
					case 1:
					break;
					
					case 2:
					IR.data|=(1<<i);
					break;
					
					default:
					IR.error=1;
					break;
				}
				_delay_us(PERIOD_RC5);
			}
			
			
			if(IR.address==RC5_ADDRESS && IR.error==0){
				switch(IR.data){
					
					case 0:
					IR.data='R';
					break;

					case 1:
					IR.data='G';
					break;

					case 2:
					IR.data='B';
					break;

					case 3:
					IR.data='V';
					break;

					case 4:
					IR.data='W';
					break;

					case 5:
					IR.data='D';
					break;

					case 6:
					IR.data='P';
					break;

					case 7:
					IR.data='E';
					break;

					case 8:
					IR.data='O';
					break;

					case 9:
					IR.data='+';
					break;

					case 10:
					IR.data='-';
					break;

					case 11:
					IR.data='*';
					break;
					
					default:
					IR.data=0xFF;
					break;
					
				}
				
				if(IR.data!=0xFF){
					UART_byte_write(IR.data);
				}
			}
			
			IR.data=0;
			IR.error=0;
			IR.address=0;
		}
	}
}

