//LED syst v1.0
//Author: Jiří Čech
//Date: 07/2015
//Target MCU: AT Mega 8A
//Clock source: external high frequency X-tal 16MHz
//Note that using AT Mega 8L is not suitable because of exceeding frequency limitation!!!
#define FIRMWARE_INFORMATION_1 "RGB CONTROLLER\n"
#define FIRMWARE_INFORMATION_2 "Firmware version: 1.0\n"
#define FIRMWARE_INFORMATION_3 "(c) Jiri Cech 01/2016\n"

//boolean values 
#define true 1
#define false 0

//Effect menu items (it's being used with void lcd_puts_effect())
#define FADE 0
#define STROBE 1
#define SOLID 2
#define NONE 3

//Menu items (it's being used with void lcd_puts_menuitem())
#define PERIOD 0
#define RC 1
#define BUZZ_BUTTON 2
#define BUZZ_RC 3
#define BUZZ_ERR 4
#define BUZZ_HELLO 5
#define E_OFF 6
#define E_ON 7
#define E_SENSE_POLL_PERIOD 8
#define E_SENSE_ENABLE 9


#define MENU_DELAY 400 //Delay of menu message in [ms]
#define LOOP_DELAY_DEFAULT 161 //Delay of main loop [unsigned char]
#define F_CPU 16000000 //CPU clock frequency is 16MHz

//Fade effect
#define FADE_DELAY 10
#define FADE_TRESHOLD 50

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#include <stdlib.h>
#include <avr/eeprom.h>
#include "i2cmaster.h"

unsigned int loop_delay; //Set number of main loop iterations (one iteration lasts 20ms)
unsigned char effect_selector; //Selector for menu of effects purpose
unsigned char animationcfg; //Configuration of animation
							//Bit_7:_-EFFECT TIMING/CYCLING-/
							//Bit_6:_-EFFECT TIMING/CYCLING-/
							//Bit_5:_-EFFECT TIMING/CYCLING-/
							//Bit_4:_-EFFECT TIMING/CYCLING-/
							//Bit_3:_Solid: 1:On; 0:Off
							//Bit_2:_Strobe: 1:On; 0:Off
							//Bit_1:_Fade: 1:On; 0:Off
							//Bit_0:_Menu: 1:Display color channels state; 0:Display welcome text
unsigned char animationcfg_previous=0; //Storage of previous value of animationcfg variable (It!s being used while handling)
														
unsigned char kbd_state; //Keyboard state |BLACK|PROPERTIES|EFFECT|WHITE|BRIGHTNESS|Light CH B|Light CH G|Light CH R| - 0xFF means error -0x00 means nothing was pressed
unsigned char renc_state; //Rotary encoder state |-|-|-|-|OK-IR_RC|OK|L|R| - 0 means rotary encoder hasn't moved
unsigned char period; //Period of effects (timing purpose)

unsigned char period_cfg; //Period of effects- modifiable in "properties" menu. 
unsigned char rc_buzz_cfg; //|light sensor EN: 1|kbd lock EN: 1|-|BUZZ BUTTON EN: 1|BUZZ ERROR EN: 1|BUZZ HELLO EN: 1|BUZZ RC EN: 1|RC EN: 1|
unsigned char e_off_cfg; //Minimal light intensity to keep lights off
unsigned char e_on_cfg; //Maximal light intensity allowed to keep lights turned on
unsigned char e_sense_poll_period_cfg; //SENSE BRDpolling interval
unsigned char e_sense_poll_period=0; //SENSE BRD polling dismiss counter
unsigned char e_value;

signed char menu_selector=0; //Screen of the main menu which is being displayed depends on the menu_selector variable. 

//Value from 0 (darkness) to 51 (full brightness) which allows to control displayed brightness and also stores value of the RGB channels while effects are being used.
unsigned char CH_R_brightness;
unsigned char CH_G_brightness;
unsigned char CH_B_brightness;


//Write byte to LCD (<MSB...LSB>, RS)
void lcd_write(unsigned char data, unsigned char rs){
	unsigned char msb_lsb_switch=0; //switching between MSB write mode(0), LSB write mode(1) and end of writing(2)
	unsigned char data_msb;
	data_msb = data & 0b11110000;
	data_msb = data_msb >> 2;
	
	data &= 0b00001111;
	data = data << 2;
	
	if (rs==true){
		data++;
		data_msb++;
	}
	
	while (msb_lsb_switch!=2){
	
	switch(msb_lsb_switch){
		case 0:
		PORTC=data_msb;
		msb_lsb_switch++;
		break;
		
		case 1:
		PORTC=data;
		msb_lsb_switch++;
		break;
	}
	
	_delay_us(10);
	PORTC|=0b00000010;
	_delay_us(10);
	PORTC&=0b11111101;
	_delay_us(45);
	}
	_delay_ms(2);
}

//Send string of chars via LCD without '\n'
void lcd_puts(char input[]){
	int i=0; //Iterator
	while(input[i] != '\n'){
		lcd_write(input[i], 1);
		i++;
	}
}

//Init LCD (PC0:RS; PC1:E; [PC2..5]:[D4..7];)
void lcd_init(){
	
	
	DDRC=0b00111111;
	_delay_us(100);

	//INIT 4 bit in 8 bit mode 8-bit 
	PORTC=0b00001000;
	_delay_us(10);
	PORTC|=0b00000010;
	_delay_us(10);
	PORTC&=0b11111101;
	_delay_us(40);
	
	lcd_write(0b00101000,0); //INIT 4 bit mode in 8 bit mode
	lcd_write(0b00000110,0); //Cursor/text move DIRECTION: R
	lcd_write(0b00001100,0); //DISP: EN; CURSOR: DIS; CURSOR_BLINK: DIS
	lcd_write(0b00000001,0); //CLR
	_delay_ms(20);
		
}

//Get information if rotary encoder has moved eventually which direction
void get_renc_state(){
		if ((PIND & (1<<PD3))) {
			if ((PIND & (1<<PD6))){
				renc_state=0b00000001;
				}else{
				renc_state=0b00000010;
			}
			
		}
		
		else {
			if ((PIND & (1<<PD6))){
				renc_state=0b00000010;
				}else{
				renc_state=0b00000001;
			}
			
		}
}

//Send byte via UART
void UART_putchar(char input){
	while (!(UCSRA & (1 << UDRE) ) ) { //Wait while buffer is empty
	}
	UDR = input;
}

//Send string of chars via UART without '\n'
void UART_puts(char input[]){
	int i=0; //Iterator
	while(input[i] != '\n'){
		UART_putchar(input[i]);
		i++;
	}
	UART_putchar('\n');
}

//Get char from UART
//Returns NULL if parity error occurred
char UART_getchar(){
	while (!(UCSRA & (1 << RXC) ) ) { //Wait while character is received
	}
	
	return UDR;
}

//Turn buzzer of the sense board on for short time (0) or long time (any value except 0 as an argument)
void buzzer(unsigned char input){
	if(input!=0){
		UART_putchar('/'); //Turn on buzzer for long time
		}else{
		UART_putchar('%'); //Turn on buzzer for short time
	}
}

//Get keyboard state.
//(Only if keyboard is not locked)
void get_kbd_state_value(){
	if((rc_buzz_cfg&0b01000000)==0){
		// read kbd state
		i2c_start(0b01000001);
		kbd_state=i2c_readAck();
		if (kbd_state!=false){
			_delay_ms(20);
			if(kbd_state!=i2c_readAck()){
				kbd_state=0xFF;
			}
			i2c_stop();
			
			if(((rc_buzz_cfg&0b00010000)!=0)&&kbd_state!=0xFF){ //If is "BUTTON" buzzer enabled
				buzzer(0); //Turn buzzer on for short time
			}
		}
	}else{
		_delay_ms(25); //Delay instead of code in previous condition
	}
}


//Display value of RED color channel
void Display_CHR(){
	lcd_write(0b11000000,0); //LCD_DDRAM_ADDR_40h
	lcd_write(((CH_R_brightness / 10) |0b00110000),1);
	lcd_write(((CH_R_brightness % 10) |0b00110000),1);
}

//Display value of GREEN color channel	
void Display_CHG(){
	lcd_write(0b11000011,0); //LCD_DDRAM_ADDR_43h
	lcd_write(((CH_G_brightness / 10) |0b00110000),1);
	lcd_write(((CH_G_brightness % 10) |0b00110000),1);
}

//Display value of BLUE color channel	
void Display_CHB(){
	lcd_write(0b11000110,0); //LCD_DDRAM_ADDR_46h
	lcd_write(((CH_B_brightness / 10) |0b00110000),1);
	lcd_write(((CH_B_brightness % 10) |0b00110000),1);
}

//Display static text for menu of color channels and values of mentioned channels
void DisplayStatic_CHx(){
	lcd_write(0b00000001,0); //LCD_CLEAR
	lcd_write(0b10000000,0); //LCD_DDRAM_ADDR_00h
	lcd_write('R',1);
	lcd_write(0b10000011,0); //LCD_DDRAM_ADDR_03h
	lcd_write('G',1);
	lcd_write(0b10000110,0); //LCD_DDRAM_ADDR_06h
	lcd_write('B',1);
	
	Display_CHR();
	Display_CHG();
	Display_CHB();
}	

//Display text on the LCD according effect which is set
void lcd_puts_effect(unsigned char input){
	lcd_write(0b00000001,0);
	lcd_write(0b10000001,0);
	lcd_puts("EFFECT\n");
	lcd_write(0b11000001,0);
	
switch(input){
	case FADE:
	lcd_puts("FADE\n");
	break;
	
	case STROBE:
	lcd_puts("STROBE\n");
	break;
	
	case SOLID:
	lcd_puts("SOLID\n");
	break;
	
	case NONE:
	lcd_puts("NONE\n");
	break;
	
}

}

void lcd_puts_menuitem(unsigned char item_number){
lcd_write(0b00000001,0); //Clear the LCD and move carriage to 00h
switch(item_number){
	case PERIOD:
	lcd_puts("PERIOD:\n");
	break;
	
	case RC:
	lcd_puts("REMOTE\n");
	lcd_write(0b11000000,0); //DDRAM ADDR 40h
	lcd_puts("CTRL:\n");
	break;
	
	case BUZZ_BUTTON:
	lcd_puts("BUZZER\n");
	lcd_write(0b11000000,0); //DDRAM ADDR 40h
	lcd_puts("BUTT:\n");
	break;
	
	case BUZZ_RC:
	lcd_puts("BUZZER\n");
	lcd_write(0b11000000,0); //DDRAM ADDR 40h
	lcd_puts("RC:\n");
	break;
	
	case BUZZ_ERR:
	lcd_puts("BUZZER\n");
	lcd_write(0b11000000,0); //DDRAM ADDR 40h
	lcd_puts("ERROR:\n");
	break;
	
	case BUZZ_HELLO:
	lcd_puts("BUZZER\n");
	lcd_write(0b11000000,0); //DDRAM ADDR 40h
	lcd_puts("HELLO:\n");
	break;
	
	case E_OFF:
	lcd_puts("E_OFF:\n");
	break;
	
	case E_ON:
	lcd_puts("E_ON:\n");
	break;
	
	case E_SENSE_POLL_PERIOD:
	lcd_puts("RC POLL:\n");
	break;
	
	case E_SENSE_ENABLE:
	lcd_puts("LIGHT\n");
	lcd_write(0b11000000,0); //DDRAM ADDR 40h
	lcd_puts("SENSE:\n");
	break;
}
}

//Clear the LCD, display static text and display value of color channels
void update_all_CHx(){
	DisplayStatic_CHx();
	Display_CHR();
	Display_CHG();
	Display_CHB();
}

//Display welcome text to the LCD
void DisplayStatic_ledsysthello(){
	lcd_write(0b00000001, 0); //Clear the LCD and set cursor to the 00h ADDR (position 0,0)
	lcd_puts("RGB_DRVR\n");
	
	lcd_write(0b11000001,0);//DDRAM ADDR 41h
	lcd_puts("HELLO!\n");
}	
//Rotary encoder OK button state ('OK' pressed: true; free: false)
unsigned char renc_ok_state(){
	unsigned char result=false;
	//unsigned char tmp;
	
	if ((PIND & (1<<PIND7))==false){
		_delay_ms(20);
		if ((PIND & (1<<PIND7))==false){
			result = true;
			if((rc_buzz_cfg&0b00010000)!=0){ //If is BUZZ BUTT enabled
				buzzer(0);
			}
		}
	}
	//tmp = renc_state>>2;
	//result|=tmp;
	
	
	renc_state&=0b11111011; //Erase 'OK' pressed flag
	
	return result;
}


//Update values in TCR registers
void channels_update(){
	OCR1A=CH_R_brightness * 5;
	OCR1B=CH_G_brightness * 5;
	OCR2=CH_B_brightness * 5;
	
	if(CH_B_brightness==0){
		DDRB&=0b11110111;
		}else{
		DDRB|=0b00001000;
	}
	
	if(CH_G_brightness==0){
		DDRB&=0b11111011;
		}else{
		DDRB|=0b00000100;
	}
	
	if(CH_R_brightness==0){
		DDRB&=0b11111101;
		}else{
		DDRB|=0b00000010;
	}
}

//EFFECT STROBE
void strobe(){
	
if(((animationcfg&0b00000100)==0b00000100) && (e_value!=0xFF)){	
	//Enable all outputs
	DDRB|=0b00001110;
	
	//Disable PWM
	TCCR1A=0x00;
	TCCR1B=0x00;
	TCCR2=0x00;
	
	if(period!=0){
		period--;
	}else{
		period=period_cfg;
	//Enable all color channels
	PORTB|=0b00001110;
	_delay_ms(1);	
	//Disable all color channels
	PORTB&=0b11110001;
	}
		
}else if((e_value!=0xFF) || ((OCR1A!=0) || (OCR1B!=0) || (OCR2!=0))){ //Enable PWM if unblocked and needed

//TCR1: 8-bit Fast PWM (TOP = 0x00FF); f=62,5kHz@16MHz fcpu (fcpu/256); OUT A(PB1), B(PB2): Non Inverted mode
TCCR1A=0b10100001; //CH_R
TCCR1B=0b00001100; //CH_G

//TCR2: 8-bit Fast PWM (TOP = 0xFF); f=62,5kHz@16MHz fcpu (fcpu/256); OUT(PB3): Non Inverted mode
TCCR2=0b01101110; //CH_B

}
}

//EFFECT SOLID
void solid(){
if(e_value!=0xFF){
if((animationcfg&0b00001000)==0b00001000){
	if(period!=0){
		period--;
		_delay_ms(1);
	}else{
period=period_cfg;//Load configured period value
DDRB|=0b00001110; //Enable outputs
switch(animationcfg&0b11111110){
	case 0b00001000:
	OCR1A=0xff;
	OCR1B=0x90;
	OCR2=0x12;
		animationcfg=0b00011000;
	break;
	
	case 0b00011000:
	OCR1A=0x02;
	OCR1B=0x15;
	OCR2=0x26;
		animationcfg=0b00101000;
	break;
	
	case 0b00101000:
	OCR1A=0x01;
	OCR1B=0xef;
	OCR2=0xaa;
		animationcfg=0b00111000;
	break;
	
	case 0b00111000:
	OCR1A=0x69;
	OCR1B=0x85;
	OCR2=0x12;
		animationcfg=0b01001000;

	break;
	
	case 0b01001000:
	OCR1A=0x25;
	OCR1B=0x89;
	OCR2=0x11;
		animationcfg=0b01011000;

	break;
	
	case 0b01011000:
	OCR1A=0xde;
	OCR1B=0xd1;
	OCR2=0xd0;
		animationcfg=0b01111000;

	break;
	
	case 0b01111000:
	OCR1A=0x10;
	OCR1B=0x2f;
	OCR2=0x33;
		animationcfg=0b10001000;

	break;
	
	case 0b10001000:
	OCR1A=0x10;
	OCR1B=0x9f;
	OCR2=0x10;
		animationcfg=0b10011000;

	break;
	
	case 0b10011000:
	OCR1A=0x88;
	OCR1B=0x74;
	OCR2=0xa1;
		animationcfg=0b10101000;

	break;
	
	case 0b10101000:
	OCR1A=0x11;
	OCR1B=0x27;
	OCR2=0x02;
		animationcfg=0b10111000;
		
	break;
	
	case 0b10111000:
	OCR1A=0x16;
	OCR1B=0xee;
	OCR2=0x45;
		animationcfg=0b11001000;

	break;
	
	case 0b11001000:
	OCR1A=0x50;
	OCR1B=0x2a;
	OCR2=0xa1;
		animationcfg=0b11011000;
	
	break;
	
	case 0b11011000:
	OCR1A=0x12;
	OCR1B=0xef;
	OCR2=0xa6;
		animationcfg=0b11101000;
	
	break;
	
	case 0b11101000:
	OCR1A=0x03;
	OCR1B=0x2a;
	OCR2=0x10;
		animationcfg=0b11111000;

	break;
	

	case 0b11111000:
	OCR1A=0xa0;
	OCR1B=0xaa;
	OCR2=0x14;
		animationcfg&=0b00001000;
	break;
}
}
}
}
}

//EFFECT FADE
void fade(){
	if(e_value!=0xFF){
	if((animationcfg&0b00000010)==2){
		if(period!=period_cfg){
			period++;
		}else{
			period=0;
		
		
		DDRB|=0b00001110; //Enable outputs	
	
	switch(animationcfg&0b11111110){
		case 0b00000010:
		OCR1A=0xFF;
		OCR1B=0x00;
		OCR2=0x00;
		animationcfg=0b00010010;
		break;
		
		case 0b00010010:
		OCR1A--;
		if(OCR1A==FADE_TRESHOLD){
			animationcfg=0b00100010;
		}
		break;
		
		case 0b00100010:
		OCR1B++;
		if(OCR1B==0xFF){
			animationcfg=0b00110010;
		}
		break;
		
		case 0b00110010:
		OCR2++;
		if(OCR2==0xFF){
			animationcfg=0b01000010;
		}
		break;
		
		case 0b01000010:
		OCR2--;
		if(OCR2==FADE_TRESHOLD){
			animationcfg=0b01010010;
		}
		break;
		
		case 0b01010010:
		OCR1A++;
		if(OCR1A==0xFF){
			animationcfg=0b01110010;
		}
		break;
		
		case 0b01110010:
		OCR1B--;
		if(OCR1B==FADE_TRESHOLD){
			animationcfg=0b10000010;
		}
		break;
		
		case 0b10000010:
		OCR2++;
		if(OCR2==0xFF){
			animationcfg=0b10010010;
		}
		break;
		
		case 0b10010010:
		OCR1A--;
		if(OCR1A==FADE_TRESHOLD){
			animationcfg=0b10100010;
		}
		break;
		
		case 0b10100010:
		OCR2--;
		if(OCR2==FADE_TRESHOLD){
			animationcfg=0b10110010;
		}
		break;
		
		case 0b10110010:
		OCR1B++;
		if(OCR1B==0xFF){
			animationcfg=0b11000010;
		}
		break;
		
		case 0b11000010:
		OCR1A++;
		if(OCR1A==0xFF){
			animationcfg=0b11010010;
		}
		break;
		
		case 0b11010010:
		OCR1B--;
		if(OCR1B==FADE_TRESHOLD){
			animationcfg=0b11100010;
		}
		break;
		
		case 0b11100010:
		OCR1A--;
		if(OCR1A==FADE_TRESHOLD){
			animationcfg=0b11110010;
		}
		break;
		

		case 0b11110010:
		animationcfg&=0b00000010;
	
		break;
	
	}
		}
	}
	}
}


//Execute menu
void menu(){
	//Execute menu
	switch(kbd_state){
		
		case 0b11111110: //Light CH R
		DisplayStatic_CHx();
		while(renc_ok_state()!=true && ((renc_state&0b00001000)==0)){
			if(CH_R_brightness<51 && renc_state==2){
				CH_R_brightness++;
				}else if(CH_R_brightness>0 && renc_state==1){
				CH_R_brightness--;
			}
			renc_state=false;
					
			OCR1AL = CH_R_brightness * 5;
			if(OCR1AL==0){
				DDRB&=0b11111101;
				}else{
				DDRB|=0b00000010;
			}
			Display_CHR();
			
		}
		renc_state=false;
		break;
		
		
		case 0b11111101: //Light CH G
		DisplayStatic_CHx();
		while(renc_ok_state()!=true && ((renc_state&0b00001000)==0)){
		
			if(CH_G_brightness<51 && renc_state==2){
				CH_G_brightness++;
				}else if(CH_G_brightness>0 && renc_state==1){
				CH_G_brightness--;
			}
			renc_state=false;
			
			OCR1BL = CH_G_brightness * 5;
			if(OCR1BL==false){
				DDRB&=0b11111011;
				}else{
				DDRB|=0b00000100;
			}
			Display_CHG();
			
		}
		renc_state=false;
		break;
		
		case 0b11111011: //LIGHT CH B
		DisplayStatic_CHx();
		while(renc_ok_state()!=true && ((renc_state&0b00001000)==0)){
			if(CH_B_brightness<51 && renc_state==2){
				CH_B_brightness++;
				}else if(CH_B_brightness>0 && renc_state==1){
				CH_B_brightness--;
			}
			renc_state=false;
			
			OCR2 = CH_B_brightness * 5;
			if(OCR2==false){
				DDRB&=0b11110111;
				}else{
				DDRB|=0b00001000;
			}
			Display_CHB();
		}
		renc_state=false;
		break;
		
		case 0b11110111: //Light level
		lcd_write(0b00000001,0);
		//Assign same values to registers of current channel's color level in case of mismatch.
		if(CH_R_brightness!=CH_G_brightness || CH_G_brightness!=CH_B_brightness || CH_B_brightness!=CH_R_brightness){
			CH_R_brightness=20;
			CH_G_brightness=20;
			CH_B_brightness=20;
			
			OCR1A=100;
			OCR1B=100;
			OCR2=100;
		}
		lcd_write(0b00000001, 0); //Clear the LCD
		lcd_puts("LIGHT\n");
		lcd_write(0b11000000, 0); //DDRAM_ADDR_40h
		lcd_puts("LEVEL:\n");
		
		//All values of color channels are increasing/ decreasing synchronously.
		//Only variable <CH_R_brightness> is changing at the moment. Other variables will change later.
		while(renc_ok_state()!=true && ((renc_state&0b00001000)==0)){
			asm volatile("cli"); //<renc_state> value won't change from now
			if(CH_R_brightness<51 && renc_state==2){
				CH_R_brightness++;
				}else if(CH_R_brightness>0 && renc_state==1){
				CH_R_brightness--;
			}
			renc_state=false; //Erase RENC state
			
			//Enable or disable outputs or color channels depending to variable <CH_R_brightness>
			if (CH_R_brightness==0){
				DDRB&=0b11110001;
				}else{
				DDRB|=0b00001110;
			}
			
			//Values of all color channels will be synchronized.
			CH_G_brightness=CH_R_brightness;
			CH_B_brightness=CH_R_brightness;
			
			//Write values to registers.
			//Here isn't used only variable <CH_R_brightness>, because it may help user with diagnostic of memory error.
			//This error is possible, but rare.
			
			channels_update();
			renc_state=false;
			
			//LCD_DDRAM_ADDR_46h
			lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
			lcd_write(((CH_R_brightness / 10)|0b00110000),1);
			lcd_write(((CH_R_brightness % 10)|0b00110000),1);
			
			asm volatile("sei"); //Interrupt system activated...
		}
		
		_delay_ms(MENU_DELAY);
		update_all_CHx();
		
		renc_state=false;
		break;
		
		case 0b11101111: //WHITE
		animationcfg=0;//Turn off light effects
		lcd_write(0b00000001,0); //Clear the LCD
		lcd_puts("WHITE\n");
		
		CH_R_brightness=51;
		CH_G_brightness=51;
		CH_B_brightness=51;
		
		channels_update();
		
		_delay_ms(MENU_DELAY);
		update_all_CHx();	
		kbd_state=0xFF;
		break;
		
		case 0b11011111: //EFFECT	
		kbd_state=0xFF;
		while((renc_ok_state())==false && ((renc_state&0b00001000)==0)){
		
			lcd_write(0b11000000,0); //LCD_ADDR_40h
			_delay_ms(50);
			
			lcd_puts_effect(effect_selector);
			
			if(kbd_state==0b11011111){
			kbd_state=0xFF;
					if(renc_ok_state()!=true){
					if(effect_selector<3){
						effect_selector++;
					}else{
						effect_selector=0;
					}
					_delay_ms(MENU_DELAY);
					}
		}
			}
			renc_state=false;

		
		
		switch(effect_selector){
			case FADE:
			animationcfg=0b00000010; //FADE ON
			break;
			
			case STROBE:
			animationcfg=0b11100100; //STROBE ON with cfg of color: white (R, G, B)-->ON
			break;
			
			case SOLID:
			animationcfg=0b00001000; //SOLID ON
			break;
			
			case NONE:
			animationcfg=0b00000000; //NONE ON
			channels_update(); //Restore value of color channels
			break;
			
		}
		animationcfg_previous=animationcfg;
		renc_state=false;
		break;
		
		case 0b10111111: //MENU
			menu_selector=-1; //Anytime start at first menu item
			while((renc_ok_state())!=true && ((renc_state&0b00001000)==0)){
			lcd_write(0b11000000,0); //LCD_ADDR_40h
			_delay_ms(50);
			if(kbd_state==0b10111111){
				kbd_state=0xFF;
					if(menu_selector<9){
						menu_selector++;
						}else{
						menu_selector=0;
					
					}
				}
					_delay_ms(MENU_DELAY);

		lcd_puts_menuitem(menu_selector);

		switch(menu_selector){
			case PERIOD:
				while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
					
				lcd_write(0b11000000,0); //Set DDRAM ADDR to 40h

				//Display period value on the LCD
				lcd_write(((period_cfg / 100) |0b00110000),1);
				lcd_write((((period_cfg % 100)/10) |0b00110000),1);
				lcd_write(((period_cfg % 10) |0b00110000),1);

				if(period_cfg<0xFF && renc_state==2){
					period_cfg++;
					}else if(period_cfg>0 && renc_state==1){
					period_cfg--;
				}
				renc_state&=0b00001100; //Erase any flags except "OK"
				}
			break;
			
		
			case RC:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			if((rc_buzz_cfg&0b00000001)==false){ //If is boolean value false...
			if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
			renc_state&=0b00000100; //Erase any flags except "OK"
			rc_buzz_cfg|=0b00000001;
			}
			lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
			lcd_puts(" x\n");
			}else{  //If is boolean value true...
			if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
			renc_state&=0b00000100; //Erase any flags except "OK"
			rc_buzz_cfg&=0b11111110;
			}
			lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
			lcd_puts("on\n");
			}
			}
			break;
			
			case BUZZ_BUTTON:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			if((rc_buzz_cfg&0b00010000)==false){ //If is boolean value false...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg|=0b00010000;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts(" x\n");
				}else{  //If is boolean value true...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg&=0b11101111;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts("on\n");
			}
			}
			break;
			
			case BUZZ_RC:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			if((rc_buzz_cfg&0b00000010)==false){ //If is boolean value false...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg|=0b00000010;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts(" x\n");
				}else{  //If is boolean value true...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg&=0b11111101;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts("on\n");
			}
			}
			break;

			case BUZZ_ERR:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			if((rc_buzz_cfg&0b00001000)==false){ //If is boolean value false...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg|=0b00001000;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts(" x\n");
				}else{  //If is boolean value true...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg&=0b11110111;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts("on\n");
			}
			}
			break;

			case BUZZ_HELLO:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			if((rc_buzz_cfg&0b00000100)==false){ //If is boolean value false...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg|=0b00000100;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts(" x\n");
				}else{  //If is boolean value true...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg&=0b11111011;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts("on\n");
			}
			}
			break;
			
			case E_OFF:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			lcd_write(0b11000000,0); //Set DDRAM ADDR to 40h
			//Display period value on the LCD
			lcd_write(((e_off_cfg / 100) |0b00110000),1);
			lcd_write((((e_off_cfg % 100)/10) |0b00110000),1);
			lcd_write(((e_off_cfg % 10) |0b00110000),1);
			
			if(e_on_cfg<0xFF && renc_state==2){ 
				e_off_cfg++;
				}else if(e_off_cfg>0 && renc_state==1){
				e_off_cfg--;		
				}
				renc_state&=0b00000100; //Erase any flags except "OK"	
			}
			break;

			case E_ON:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			lcd_write(0b11000000,0); //Set DDRAM ADDR to 40h

			//Display period value on the LCD
			lcd_write(((e_on_cfg / 100) |0b00110000),1);
			lcd_write((((e_on_cfg % 100)/10) |0b00110000),1);
			lcd_write(((e_on_cfg % 10) |0b00110000),1);

			if(e_on_cfg<0xFE && renc_state==2){
				e_on_cfg++;
				}else if(e_on_cfg>0 && renc_state==1){
				e_on_cfg--;
			}
				renc_state&=0b00000100; //Erase any flags except "OK"
			}
			break;

			case E_SENSE_POLL_PERIOD:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			lcd_write(0b11000000,0); //Set DDRAM ADDR to 40h

			//Display period value on the LCD
			lcd_write(((e_sense_poll_period_cfg / 100) |0b00110000),1);
			lcd_write((((e_sense_poll_period_cfg % 100)/10) |0b00110000),1);
			lcd_write(((e_sense_poll_period_cfg % 10) |0b00110000),1);

			if(e_sense_poll_period_cfg<0xFF && renc_state==2){
				e_sense_poll_period_cfg++;
				e_sense_poll_period=0; //Reset polling timer
				}else if(e_sense_poll_period_cfg>0 && renc_state==1){
				e_sense_poll_period_cfg--;
				e_sense_poll_period=0; //Reset polling timer
			}
			renc_state&=0b00000100; //Erase any flags except "OK"
			}
			break;
			
			case E_SENSE_ENABLE:
			while((renc_ok_state())!=true && (kbd_state!=0b10111111) && ((renc_state&0b00001000)==0)){
			if((rc_buzz_cfg&0b10000000)==false){ //If is boolean value false...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					renc_state&=0b00000100; //Erase any flags except "OK"
					rc_buzz_cfg|=0b10000000;
				}
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts(" x\n");
				}else{  //If is boolean value true...
				if((renc_state&0b00000011)!=false){ //If RENC has moved in any direction
					
					rc_buzz_cfg&=0b01111111;
				}
				renc_state&=0b00000100; //Erase any flags except "OK"
				lcd_write(0b11000110, 0); //DDRAM_ADDR_46h
				lcd_puts("on\n");
				}
			}
			break;	
			}
			}
			break;
		
		case 0b01111111: //BLACK
		rc_buzz_cfg&=0b01111111; //Disable light sensor
		animationcfg=0;//Turn off light effects
		lcd_write(0b00000001,0); //Clear the LCD
		lcd_puts("BLACK\n");
		
		CH_R_brightness=0;
		CH_G_brightness=0;
		CH_B_brightness=0;
		
		OCR1A=0;
		OCR1B=0;
		OCR2=0;
		DDRB&=0b11110001; //Disable outputs
		_delay_ms(MENU_DELAY);
		update_all_CHx();		
		kbd_state=0xFF;
		break;
		
		case 0xFF:
		break;
		
		default: //In case of fault (more than one button on the kbd has been pressed)
		if((rc_buzz_cfg&0b00001000)!=0){ //*beep* if ERROR buzzer is enabled
		buzzer(1); //Long *beep*
		}
		break;

}
renc_state=false; //Erase any renc/ ok/ RC ok states
}


			
//INT0 ISR (I/O I2C expander) 
ISR(INT0_vect){
_delay_ms(1);
get_kbd_state_value();
}

//INT1 ISR (rotary encoder)
ISR(INT1_vect){
	_delay_us(300);
	get_renc_state(); 
	
}

//UART Rx complete 
ISR(USART_RXC_vect){
unsigned char c=UART_getchar(); //Save received character
_delay_ms(5); 
if((rc_buzz_cfg&0b00000001)!=0){ //If RC is enabled...

if((rc_buzz_cfg&0b00000010)!=0){ //If "RC" buzzer is enabled
	buzzer(0); //Turn buzzer on for short time
}	

switch(c){
	case 'R': //RED
	kbd_state=0b11111110;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'G': //GREEN
	kbd_state=0b11111101;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'B': //Blue
	kbd_state=0b11111011;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'V': //LIGHT LEVEL
	kbd_state=0b11110111;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'W': //White
	kbd_state=0b11101111;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'E': //EFFECT
	kbd_state=0b11011111;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'P': //Properties
	kbd_state=0b10111111;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'D': //Black
	kbd_state=0b01111111;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case 'O': //OK
	renc_state|=0b00001000; //'OK' Command comes from the IR RC
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case '-': //DECREMENT
	renc_state|=0b00000001;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case '+': //INECREMENT
	renc_state|=0b00000010;
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer s enabled...
		buzzer(0); //Short *beep*
	}
	break;
	
	case '?': //View firmware information
	UART_puts("\n");
	UART_puts("-\n");
	UART_puts(FIRMWARE_INFORMATION_1);
	UART_puts(FIRMWARE_INFORMATION_2);
	UART_puts(FIRMWARE_INFORMATION_3);
	UART_puts("-\n");

	buzzer(0); //Short *beep*
	_delay_ms(200);
	buzzer(0); //Short *beep*
	_delay_ms(200);
	buzzer(0); //Short *beep*
	_delay_ms(200);	
	buzzer(0); //Short *beep*
	break;
	
	case '*': //Lock/ unlock device keyboard
	if((rc_buzz_cfg&0b00000010)!=0){ //If RC buzzer is enabled...
		buzzer(0); //Short *beep*
	}
	lcd_write(0b00000001,0); //Clear the LCD and set the cursor position to 00h
	lcd_puts("KEYBOARD\n");
	
	if((rc_buzz_cfg&0b01000000)==0){
		rc_buzz_cfg|=0b01000000; //Lock kbd if unlocked
		kbd_state=0xFF; //Clear kbd state value
		lcd_write(0b11000000,0); //LCD_DDRAM_ADDR_40h
		lcd_puts("LOCKED\n");
	}else{
		rc_buzz_cfg&=0b10111111; //Unlock kbd if locked
		lcd_write(0b11000000,0); //LCD_DDRAM_ADDR_40h
		lcd_puts("UNLOCKED\n");
	}
	_delay_ms(MENU_DELAY);
	lcd_write(0b00000001,0); //Clear the LCD and set the cursor position to 00h
	DisplayStatic_ledsysthello();
	break;
	
}	
}
}

int main(void)
{

	eeprom_busy_wait();
	if(eeprom_read_byte((uint8_t*)10)==0xFF){ //Load default configuration if device is turned on first time
		
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t*)0,0b00010111); //rc_buzz_cfg
		
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t*)1,10); //period_cfg
		
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t*)2,150); //e_on_cfg
		
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t*)3,160); //e_off_cfg
		
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t*)4,30); //e_sense_poll_cfg
		
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t*)10,0); //Default configuration has been loaded
		
	}
	
	//Restore configuration from the EEPROM
	eeprom_busy_wait();
	rc_buzz_cfg=eeprom_read_byte((uint8_t*)0);
	
	period_cfg=eeprom_read_byte((uint8_t*)1);
	
	e_on_cfg=eeprom_read_byte((uint8_t*)2);
	
	e_off_cfg=eeprom_read_byte((uint8_t*)3);
	
	e_sense_poll_period_cfg=eeprom_read_byte((uint8_t*)4);

	//Initialization
	//Interrupt
	//INT0: falling edge + pull-up R
	//INT1: falling edge 
	PORTD|=0b00000100;
	GICR=0b11000000;
	MCUCR|=0b00001010;
	
	//UART: 9600-8-1-N; RxC INT, 
	UCSRB=0b10011000;
	UCSRC=0b10000110;
	UBRRL=0b01100111;
	UBRRH=0b00000000;
	
	//Initialization of TCRs as PWM is in "else" after main condition of strobe effect
	_delay_ms(500);	 
	lcd_init();
				 
	if((rc_buzz_cfg&0b00000100)!=0){ //If is "HELLO" buzzer enabled for long time
		buzzer(1);
	}
	
	//Global enable interrupts
	sei();
    while(1)
    {
			//UPDATE data stored in the EEPROM if the data has changed
			if(eeprom_read_byte((uint8_t*)0)!=rc_buzz_cfg){ //Check if is configuration in the EEPROM up to date
				eeprom_busy_wait();
				eeprom_write_byte((uint8_t*)0,rc_buzz_cfg);
			}
			
			if(eeprom_read_byte((uint8_t*)0)!=period_cfg){ //Check if is configuration in the EEPROM up to date
				eeprom_busy_wait();
				eeprom_write_byte((uint8_t*)1,period_cfg);
			}
			
			if(eeprom_read_byte((uint8_t*)0)!=e_on_cfg){ //Check if is configuration in the EEPROM up to date
				eeprom_busy_wait();
				eeprom_write_byte((uint8_t*)2,e_on_cfg);
			}
			
			if(eeprom_read_byte((uint8_t*)0)!=e_off_cfg){ //Check if is configuration in the EEPROM up to date
				eeprom_busy_wait();
				eeprom_write_byte((uint8_t*)3,e_off_cfg);
			}
			
			if(eeprom_read_byte((uint8_t*)0)!=e_sense_poll_period_cfg){ //Check if is configuration in the EEPROM up to date
				eeprom_busy_wait();
				eeprom_write_byte((uint8_t*)4,e_sense_poll_period_cfg);
			}
			
			switch(animationcfg&0b00001111){ //LCD animation switch
				
				case 0b00000001:
				DisplayStatic_CHx();
				animationcfg--;
				break;
				
				case 0b00000011:
				lcd_puts_effect(FADE);
				animationcfg--;
				break;
				
				case 0b00000101:
				lcd_puts_effect(STROBE);
				animationcfg--;
				break;
				
				case 0b00001001:
				lcd_puts_effect(SOLID);
				animationcfg--;
				break;
				
				default:
				DisplayStatic_ledsysthello();
				animationcfg++;
				break;
			}
					
									
		for(loop_delay=LOOP_DELAY_DEFAULT;loop_delay!=0;loop_delay--){ //Main loop animation timer
		//----Main loop----
			cli();
			get_kbd_state_value();
			sei();
			
			fade();	
			strobe();
			solid();
			menu();
			
			if((rc_buzz_cfg&0b10000000)!=0){ //If light sensor is enabled
				if(e_sense_poll_period!=e_sense_poll_period_cfg){ //Polling timer
					e_sense_poll_period++;
				}else{
				e_sense_poll_period=0; //Reset polling timer
				asm("cli");
				UART_putchar('!');
				_delay_ms(5);
				e_value=UDR;
				UART_putchar('#');
				_delay_ms(5);
				e_value=UDR;
				if(e_value<=e_on_cfg){
					DDRB|=0b00001110;				
					e_value=0; //Unblock PWM init
					}else if(e_value>=e_off_cfg){
					DDRB&=0b11110001;
					e_value=0xFF; //Block PWM init
				}
				asm("sei");
				}
				}else{
					e_value=0; //If SENSE BRD is not present then set e_value to 0 and turn the lights on
				}
				
				//channels_update(); //Update color value of color channels
		
		//----Main loop----			
		}	
}
}
		